#include <stdio.h>

#include <stdlib.h>
#include "vga.h"
#include "propeller.h"
#include "my_vga_functions.h"
#include "automatacelular.h"



uint8_t grid[2][GRID_X][GRID_Y];
uint8_t current_grid = 0;

int viven[MAXLONGREGLA];
int nacen[MAXLONGREGLA];

void imprimeReglas()
{
    int i;
    int x = 75;
    
    printf("\nViven: ");
    for(i=0; i < MAXLONGREGLA - 1; i++){
        printf("%d ", viven[i]);
        printdigit( viven[i], x+=7 , 5, Blue);
        }
     
    x = 75;
    printf("\nNacen: ");  
    for(i=0; i < MAXLONGREGLA - 1; i++){
        printf("%d ", nacen[i]);
        printdigit( nacen[i], x+=7 , 15, Blue);
        }
}

void iniciaReglasPorDefecto()
{
    // Reglas por defecto, estas reglas son distintas a las de conway, pero producen unos interesantes patrones
    viven[0] = 1;
    viven[1] = 2;
    viven[2] = 3;
    viven[3] = 0;
    
    nacen[0] = 3;
    nacen[1] = 4;
    nacen[2] = 0;
    
    
    imprimeReglas();
}


/* leeReglas: lee una linea de reglas del terminal y rellena los arrays de viven y nacen
El formato de introduccion de las reglas es p.ej "23/3" que significa que ...
Una célula viva con 2 ó 3 células vecinas vivas sigue viva, en otro caso muere
Una célula muerta con exactamente 3 células vecinas vivas "nace" 
 */
int leeReglas()
{
    char c, i,j;
    char num;
    
    //Llenando el array viven
    for(i = 0; i < MAXLONGREGLA - 1 && (c = getchar()) != '/' && c != '\n'; ++i){
         //Para la conversion de char a int hay que tener en cuenta el codigo ASCII y restarle 48
        if((num = c - 48) > 0 && num < 9){          
            viven[i] = num;
        }else{
            //Si el formato de entrada de las reglas no es correcto se inician las reglas por defecto
            iniciaReglasPorDefecto();   
            return 1;
            }
        }
    //Llenando el array nacen
    for(j = 0; j < MAXLONGREGLA - 1 && (c = getchar()) != ' ' && c != '\n'; ++j){
        if((num = c - 48) > 0 && num < 9){
            nacen[j] = num;
        }else{
            //Si el formato de entrada de las reglas no es correcto se inician las reglas por defecto
            iniciaReglasPorDefecto();
            return 1;
            }
        }
    
    //Si no se han introducido ninguna regla...:
    if(i == 0 && j == 0){
        iniciaReglasPorDefecto();
        return 1;
        }
    
   imprimeReglas();
        
    return 0;
   
}

/* El formato de entrada de la configuración será el de una sola linea de texto con puntos y asteriscos , separadores 
de linea '$', y marcador final 'f'. 
Todo caracter que no sea el "." sera considerado una celula viva y el "." sera una celula muerta
Por ejemplo:   "..*$***$..*\n"  donde la primera linea es ..* la segunda *** y la tercera ..*  

NOTA: el terminal tiene problemas cuando hacemos un copia y pega de las configuraciones */
int leeConfiguracionTerminal(){
    
    current_grid = 0;
    int lim = GRID_X;
    int x = 0;
    int y = 0;
    
    char c;
    
    do{
        y++;
        x = 0;
        lim = GRID_X;
        while(--lim > 0 && (c = getchar()) != 'f' && c != '$'){
            if(c != '.')
                grid[current_grid][++x][y] = 1;
            else 
                grid[current_grid][++x][y] = 0;
            }
            
    }while(c != 'f' && lim > 0);
  
  return 0;  

}

/* Lee la configuración de un array de longitud dada*/
int leeConfiguracion(char a[], int ancho){

    if( ancho > GRID_X){
        printf("\n Configuracion demasiado grande para el tamaño de la pantalla");
        return 1;
        }
        
    current_grid = 0;
    int x = 0;
    int y = 0;
    int i = 0;
    
    char c;
    
    while((c =  a[i]) &&  (c != '\0') ){
        
        if(i++ % ancho == 0){ 
            y++;
            x = 0;
            }
            
        if(c != '.')
            grid[current_grid][++x][y] = 1;
        else 
            grid[current_grid][++x][y] = 0;
            
        }       
        
  
  return 0;  

}
uint8_t conf[8][8] = {
    {0,0,0,1,1,0,0,0},
    {0,0,1,0,0,1,0,0},
    {0,1,0,0,0,0,1,0},
    {1,0,0,0,0,0,0,1},
    {1,0,0,0,0,0,0,1},
    {0,1,0,0,0,0,1,0},
    {0,0,1,0,0,1,0,0},
    {0,0,0,1,1,0,0,0},
};
    

/* Lee la configuración de un array de longitud dada*/
int leeConfiguracion2(){

    int i,j;
    
    for(i = 0; i < 8; i++){
        for(j = 0; j < 8; j++){
             if(conf[i][j] == 1)
                grid[current_grid][i][j] = 1;
             else 
                grid[current_grid][i][j] = 0;
            }
        }
    return 0;
}


// Objetos sacados de http://en.wikipedia.org/wiki/Conway's_Game_of_Life
void inicioPrefijado()
{
    //Glider   
    grid[0][8][10] = 1;
    grid[0][9][10] = 1;
    grid[0][10][10] = 1;
    grid[0][10][9] = 1;
    grid[0][9][8] = 1;
      
    
}


int cmpGrid()
{
 int i, j;
 for (i=0; i < GRID_Y; i++) {
    for (j=0; j < GRID_X; j++) {
        if (grid[0][i][j] != grid[1][i][j]) 
            { return 0; }
    }
 }
 return 1;
}

void inicioAleatorio()
{
 int i, j;
 int t;
 blackScreen();
 current_grid = 0;
 for (i = 0; i < GRID_X; i++) {
    for (j = 0; j < GRID_Y; j++) {

        if ((uint8_t)rand() % 3 == 0) {
            grid[0][i][j] = 1;
            }
        else {
            grid[0][i][j] = 0;
            }
    }
 }
}
/* add to a width index, wrapping around like a cylinder */

inline static int xadd (int i, int a) {
    i += a;
    while (i < 0) i += GRID_X;
    while (i >= GRID_X) i -= GRID_X;
    return i;
}

/* add to a height index, wrapping around */

inline static int yadd (int i, int a) {
    i += a;
    while (i < 0) i += GRID_Y;
    while (i >= GRID_Y) i -= GRID_Y;
    return i;
}

int count_neighbours(int i, int j)
{ 
   int    k, l, count;

    count = 0;

    /* go around the cell */

    for (k=-1; k<=1; k++) for (l=-1; l<=1; l++)

        /* only count if at least one of k,l isn't zero */
        if (k || l)
              
            if (grid[current_grid][xadd(i, k)][yadd(j,l)]) count++;           
                    
    return count;
}

void runGrid()
{
 uint8_t x, y;
 int count;
 int i;
 uint8_t reglaAplicada = 0;

 uint8_t value = 0;
 uint8_t new_grid;
 new_grid = 1 - current_grid;
 
 for (y = 0; y < GRID_Y; y++) {
    for (x = 0; x < GRID_X; x++) {
        count = count_neighbours(x, y);
       
        //Aplicamos las reglas
        reglaAplicada = 0;
        for(i = 0; i < 9; i++){
            //nacimientos
            if(count != 0 && count == nacen[i]){
                value = 3;
                reglaAplicada = 1;
                break;
            }
            //supervivientes   
            if(count != 0 && count == viven[i]){
                value = grid[current_grid][x][y];
                reglaAplicada = 1;
                break;
            }
        }
        //muertes
        if(!reglaAplicada )
            value = 0;
        
        grid[new_grid][x][y] = value;
          
       }
 }
 current_grid = new_grid;
}



void drawGrid()
{
 uint8_t  x,  y;
 uint8_t cx, cy;
 uint8_t grid_next_colour = 0;
 cx = 0;
 cy = 0;
 
 for (y = 0; y < GRID_Y; y++) {
    cx = 0;
    for (x = 0; x < GRID_X; x++) {
        
        if (grid[1-current_grid][x][y] != grid[current_grid][x][y]) {
            if(grid[current_grid][x][y]) {
      
                 printPoint(  1+cx, 3+cy, White);
             
                 }                  
                            
            else 
                 printPoint(  1+cx, 3+cy, Black);
                 
                 
             }
        cx += 2;
    }
    cy += 2;
 }
 
 //Dibujo el marco 
 vline(   0, 2, 75, Blue);
 vline(   76, 2, 75, Blue);
 hline( 0, 2, 76, Blue);
 hline( 0, 76, 76, Blue);
 
 //Muestro las estadisticas
 int vivas = midoVivas();
 imprimeReglas();
 unsigned long densidad;
 densidad = (vivas * 1000) /  (NUM_CELULAS_MAX * 10);
 printf("\n total: %d", NUM_CELULAS_MAX);
 printf("\n numVivas: %d", vivas);
 printf("\n densidad: %d\n\n", densidad);
 print4digits(  vivas, 40, 110, White);
 print2digits(  densidad, 80, 110, White);
}

int midoVivas(){

    int x,y;
    int numVivas = 0;
    
    for (y = 0; y < GRID_Y; y++) {
        for (x = 0; x < GRID_X; x++) {
       
            if(grid[current_grid][x][y]) 
                numVivas++;         
            }
        }
    return numVivas;
}