#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "vga.h"
#include "propeller.h"


unsigned char framebuffer[ROWS*COLS];
char* frameptr = framebuffer;

/* Imprime un punto */
void printPoint(  int x, int y, unsigned char color)
{
    frameptr = framebuffer;
    frameptr += (y*COLS) + x;
    *frameptr++ = color;
}

/* Mira el estado de color de un punto, devuelve 1 en caso de que el color coincida o 0 en caso contrario */
int isColor(  int x, int y,unsigned char color)
{  
    frameptr = framebuffer;
    frameptr += (y*COLS) + x;
    
    return (*frameptr == color);
    
} 

/* draw a vertical line */
void vline(   int x, int y, int h, unsigned char color)
{
    frameptr = framebuffer;
    int i;
    frameptr += (y*COLS) + x;
    if (y + h >= ROWS)
        h = ROWS - y;
    for (i = 0; i < h; i++) {
        *frameptr = color;
        frameptr += COLS;
    }
}



/* draw a horizontal line */
void hline( int x, int y, int w, unsigned char color)
{
    frameptr = framebuffer;
    int i;
    frameptr += (y*COLS) + x;
    if (x + w > COLS)
        w = COLS - x;
    for (i = 0; i < w; i++)
        *frameptr++ = color;
}

/* marco: imprime un marco con el vertice superior izquierdo en las coordenadas x,y . Anchura w y altura h */
void marco( int x, int y, int w,int h, unsigned char color)
{
    vline(   x, y, h, color);
    vline(   x+ w, y, h, color);
    hline(   x, y, w, color);
    hline(   x, y + h, w, color);
    
}

/* draw a box */
/* does this using vertical rather than horizontal lines because most of
 * our shapes are higher than wide
 */
void box(   int x, int y, int w, int h, unsigned char color)
{
    frameptr = framebuffer;
    int i;
    if (x + w >= COLS)
        w = COLS - x;
    for (i = 0; i < w; i++) {
        vline( x, y, h, color);
        x++;
    }
}

/* Pantalla negra */
void blackScreen()
{
   memset(framebuffer, Black, ROWS*COLS);
}

/* print a single digit n (0-9) at position x,y, using color "color" */
/* note: no error checking here */
void printdigit( int n, int x, int y, unsigned char color)
{
 
    frameptr = framebuffer;
    static unsigned char digits[] = {
        0xf, 0x9, 0x9, 0x9, 0xf,
        0x1, 0x1, 0x1, 0x1, 0x1,
        0xf, 0x1, 0xf, 0x8, 0xf,
        0xf, 0x1, 0xf, 0x1, 0xf,
        0x9, 0x9, 0xf, 0x1, 0x1,

        0xf, 0x8, 0xf, 0x1, 0xf, 
        0xf, 0x8, 0xf, 0x9, 0xf, 
        0xf, 0x1, 0x1, 0x1, 0x1, 
        0xf, 0x9, 0xf, 0x9, 0xf, 
        0xf, 0x9, 0xf, 0x1, 0xf, 

    };
    unsigned char *ptr = digits + n*5;
    int i, j;
    unsigned c;

    frameptr += COLS*y + x;
    for (j = 0; j < 5; j++) {
        c = *ptr++;
        if (c & 0x8) frameptr[0] = color;
        if (c & 0x4) frameptr[1] = color;
        if (c & 0x2) frameptr[2] = color;
        if (c & 0x1) frameptr[3] = color;
        frameptr += COLS;
    }
}

/* print the score (a 2 digit number) at position x,y */
void print2digits(  int number, int x, int y, unsigned char color)
{
    box( x,  y, 30, 10, Black); 
    frameptr = framebuffer;
    int tens, ones;

    tens = 0;
    while (number >= 10) {
        tens++;
        number -= 10;
    }
    printdigit( tens, x, y, color);
    printdigit( number, x+5, y, color);
}


/* print the score (a 2 digit number) at position x,y */
void print3digits(  int number, int x, int y, unsigned char color)
{
    box( x,  y, 30, 10, Black);
    frameptr = framebuffer;
    int cent,tens, ones;

    cent = 0;
    while (number >= 100) {
        cent++;
        number -= 100;
    }
    
    tens = 0;
    while (number >= 10) {
        tens++;
        number -= 10;
    }
    printdigit( cent, x, y, color);
    printdigit( tens, x+5, y, color);
    printdigit( number, x+10, y, color);
}

/* print the score (a 2 digit number) at position x,y */
void print4digits(  int number, int x, int y, unsigned char color)
{
    box( x,  y, 40, 10, Black);
    frameptr = framebuffer;
    int mil,cent,tens;
    
    mil = 0;
    while (number >= 1000) {
        mil++;
        number -= 1000;
    }

    cent = 0;
    while (number >= 100) {
        cent++;
        number -= 100;
    }
    
    tens = 0;
    while (number >= 10) {
        tens++;
        number -= 10;
    }
    printdigit( mil, x, y, color);
    printdigit( cent, x+5, y, color);
    printdigit( tens, x+10, y, color);
    printdigit( number, x+15, y, color);
}

volatile struct {
    unsigned stack[16];
    struct vga_param v;
} par;


void startvga(volatile void *parptr)
{
   extern unsigned int _load_start_vga_cog[];
  
#if defined(__PROPELLER_XMM__) || defined(__PROPELLER_XMMC__)
   load_cog_driver_xmm(_load_start_vga_cog, 496, (uint32_t *)parptr);
#else
   cognew(_load_start_vga_cog, parptr);
#endif
}

void init_vga()
{
     unsigned int frequency, i, testval;
    unsigned int clkfreq = _CLKFREQ;

    _DIRA = (1<<15);
    _OUTA = 0;

    par.v.displayBuffer = framebuffer;
    par.v.directionState = (0xff << (8*PINGROUP));
    par.v.videoState = 0x300000FF | (PINGROUP<<9);
    testval = (25175000 + 1600) / 4;
    frequency = 1;
    for (i = 0; i < 32; i++) {
        testval = testval << 1;
        frequency = (frequency << 1) | (frequency >> 31);
        if (testval >= clkfreq) {
            testval -= clkfreq;
            frequency++;
        }
    }
    par.v.frequencyState = frequency;
    par.v.enabled = 1;

    memset(framebuffer, Black, ROWS*COLS);
    startvga(&par.v);
}